#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include "dib.h"
#include "wtb.h"
#include "x_util.h"

Display* dis;  // Default Display
int scr;  // Default Screen
Window rootwin;
int screen_depth;  // screen depth
GC gc;  // utility gc

int pal_mode;
char pal_file[256];

int resize(unsigned char* bits, int row, int *width, int *height)
{
	int f, y_min, y_max, x_min, x_max;
	int x, y, i, j;
	
	f = 0;
	for(y=0; y<*height; y++) {
		for(x=0; x<*width; x++) {
			if (bits[y*row + x]!=0) {
				y_min = y;
				f = 1;
				break;
			}
		}
		if (f) break;
	}
	if (f==0) return -1;
	f = 0;
	for(y=*height-1; y>=0; y--) {
		for(x=0; x<*width; x++) {
			if (bits[y*row + x]!=0) {
				y_max = y;
				f = 1;
				break;
			}
		}
		if (f) break;
	}
	if (f==0) return -1;
	f = 0;
	for(x=0; x<*width; x++) {
		for(y=0; y<*height; y++) {
			if (bits[y*row + x]!=0) {
				x_min = x;
				f = 1;
				break;
			}
		}
		if (f) break;
	}
	if (f==0) return -1;
	f = 0;
	for(x=*width - 1; x>=0; x--) {
		for(y=0; y<*height; y++) {
			if (bits[y*row + x]!=0) {
				x_max = x;
				f = 1;
				break;
			}
		}
		if (f) break;
	}
	if (f==0) return -1;
	for(i=0; i<y_max-y_min+1; i++) {
		for(j=0; j<x_max-x_min+1; j++) {
			bits[i*row + j] = bits[(y_min+i)*row + (x_min + j)];
		}
	}
	*width  = x_max - x_min + 1;
	*height = y_max - y_min + 1;
	return 0;
}


int
load_and_save(Pixmap* pm, Drawable win, char* fname, int *width, int* height)
{
	int x, y, row;
	unsigned char rgb[256*3];
	int color_used[256];
	unsigned int pixels[256];
	unsigned char* buf;
	char path[256];
	int i, w, h, d;
	
	if (pal_mode) {
		for(i=0; i<256*3; i++) rgb[i] = 0;
		if (DIBInfo(pal_file, &w, &h, &d, rgb)!=0) {
			return -1;
		}
		if (DIBInfo(fname, &w, &h, &d, NULL)!=0) {
			return -1;
		}
	} else {
		for(i=0; i<256*3; i++) rgb[i] = 0;
		if (DIBInfo(fname, &w, &h, &d, rgb)!=0) {
			return -1;
		}
	}
	buf = (unsigned char*)malloc(w*h);
	if (DIBLoad(fname, buf, w, h, w, 0, 0, NULL)!=0) {
		free(buf);
		return -1;
	}
	row = w;
	resize(buf, row, &w, &h);
	for(i=0; i<256; i++) {
		color_used[i] = 0;
	}
	for(y=0; y<h; y++) {
		for(x=0; x<w; x++) {
			color_used[buf[y*row + x]] = 1;
		}
	}
	for(i=0; i<256; i++) {
		if (color_used[i]) {
			pixels[i] = alloc_color(rgb[i*3 + 0], rgb[i*3 + 1], rgb[i*3 + 2]);
		}
	}
	*pm = XCreatePixmap(dis, win, w, h, screen_depth);
	for(y=0; y<h; y++) {
		for(x=0; x<w; x++) {
			XSetForeground(dis,gc,pixels[buf[y*row + x]]);
			XDrawPoint(dis, *pm, gc, x, h-y-1);
		}
	}
	
	strcpy(path, fname);
	for(i=strlen(path)-1; i>=0; i--) {
		if (path[i]=='.') break;
	}
	if (i>=0 && path[i]=='.') {
		path[i+1] = 'w';
		path[i+2] = 't';
		path[i+3] = 'b';
		path[i+4] = '\0';
	} else {
		strcat(path, ".wtb");
	}
	WTBSave(path, buf, w, h, row, rgb);
	free(buf);
	*width  = w;
	*height = h;
	return 0;
}


int main(int argc, char** argv)
{
	int w, h;
    Window win;
    XSetWindowAttributes att;
    XEvent ev;
	XGCValues gv;
	Pixmap pm;
	clock_t last_clk;
	int pt, f, i, j, step;

	pal_mode = 0;
	do {
		f = 0;
		for(i=1; i<argc; i++) {
			if (argv[i][0]=='-') {
				if (argv[i][1]=='p' || argv[i][1]=='P') {
					pal_mode = 1;
					if (argv[i][2]) {
						strcpy(pal_file, &argv[i][2]);
						step = 1;
					} else {
						if (i+1<argc) {
							strcpy(pal_file, argv[i+1]);
						} else {
							fprintf(stderr, "need palet file\n");
							exit(1);
						}
						step = 2;
					}
				}
				for(j=i+step; j<argc; i++,j++) {
					argv[i] = argv[j];
				}
				argc -= step;
				f = 1;
				break;
			}
		}
	} while(f);
	
	dis = XOpenDisplay(NULL);  // displayΥץ
	if(dis==NULL) {
		fprintf(stderr, "Can't Open Display\n");
		exit(1);
	}
	scr = DefaultScreen(dis);
	rootwin = DefaultRootWindow(dis);
	screen_depth = DefaultDepth(dis, scr);
	
    win = XCreateSimpleWindow(// ɥΥץ
		dis,
		rootwin,
		100, 100,
		200, 200,
		5,
		BlackPixel(dis,scr),
		WhitePixel(dis,scr));

    att.backing_store = WhenMapped;
    XChangeWindowAttributes(dis, win, CWBackingStore, &att);

	gv.graphics_exposures = False;
	gc = XCreateGC(dis, win, GCGraphicsExposures, &gv);
	XStoreName(dis, win, "bmp");
    XSelectInput(dis, win, ExposureMask | ButtonPressMask | KeyPressMask);
    XMapWindow(dis, win);
	XFlush(dis);
	do {
		XNextEvent(dis, &ev);
	} while(ev.type!=Expose);

	pm = None;
	pt = 1;
	
	last_clk = clock();
	while(1) {
		if (XCheckMaskEvent(dis,
							ExposureMask | KeyPressMask | ButtonPressMask,
							&ev)) {
			if(ev.type==KeyPress) {
				break;
			}
			else if(ev.type==ButtonPress) {
				break;
			}
			else if(ev.type==Expose) {
				if (pm!=None) {
					XCopyArea(dis, pm, win, gc, 0, 0, w, h, 0, 0);
				}
			}
		}
		else if(clock() - last_clk >= CLOCKS_PER_SEC/10) {
			last_clk = clock();
			if (pm!=None && pt<argc) {
				XFreePixmap(dis, pm);
			}
			if (pt<argc) {
				printf("%s", argv[pt]);
				if (load_and_save(&pm, win, argv[pt++], &w, &h)!=0) {
					printf(" -- error\n");
				} else {
					printf("\n");
					XResizeWindow(dis, win, w, h);
					XCopyArea(dis, pm, win, gc, 0, 0, w, h, 0, 0);
				}
			}
		}
	}

	XFreeGC(dis, gc);
	if (pm!=None) XFreePixmap(dis, pm);
	return 0;
}


