#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include "wtb.h"
#include "x_util.h"

Display* dis;  // Default Display
int scr;  // Default Screen
Window rootwin;
int screen_depth;  // screen depth
GC gc;  // utility gc

int pal_mode;
char pal_file[256];
unsigned char rgb[256*3];

int load(Pixmap* pm,
		 Drawable vis, char* fname, int* width, int* height)
{
	if (pal_mode) {
		return loadWTBWithPal(pm, vis, fname, width, height, rgb, 256);
	} else {
		return loadWTB(pm, vis, fname, width, height);
	}
}


int main(int argc, char** argv)
{
	int w, h;
    Window win;
    XSetWindowAttributes att;
    XEvent ev;
	XGCValues gv;
	Pixmap pm;
	int pt;
	int i, j, f, step, d;

	pal_mode = 0;
	do {
		f = 0;
		for(i=1; i<argc; i++) {
			if (argv[i][0]=='-') {
				if (argv[i][1]=='p' || argv[i][1]=='P') {
					pal_mode = 1;
					if (argv[i][2]) {
						strcpy(pal_file, &argv[i][2]);
						step = 1;
					} else {
						if (i+1<argc) {
							strcpy(pal_file, argv[i+1]);
						} else {
							fprintf(stderr, "need palet file\n");
							exit(1);
						}
						step = 2;
					}
				}
				for(j=i+step; j<argc; i++,j++) {
					argv[i] = argv[j];
				}
				argc -= step;
				f = 1;
				break;
			}
		}
	} while(f);

	if (pal_mode) {
		if (WTBInfo(pal_file, &w, &h, &d, rgb)!=0) {
			fprintf(stderr, "Can't Open Palette\n");
			exit(1);
		}
	}
	
	dis = XOpenDisplay(NULL);  // displayΥץ
	if(dis==NULL) {
		fprintf(stderr, "Can't Open Display\n");
		exit(1);
	}
	scr = DefaultScreen(dis);
	rootwin = DefaultRootWindow(dis);
	screen_depth = DefaultDepth(dis, scr);
	
    win = XCreateSimpleWindow(// ɥΥץ
		dis,
		rootwin,
		100, 100,
		200, 200,
		5,
		BlackPixel(dis,scr),
		WhitePixel(dis,scr));

    att.backing_store = WhenMapped;
    XChangeWindowAttributes(dis, win, CWBackingStore, &att);

	gv.graphics_exposures = False;
	gc = XCreateGC(dis, win, GCGraphicsExposures, &gv);
	XStoreName(dis, win, "bmp");
    XSelectInput(dis, win, ExposureMask | ButtonPressMask | KeyPressMask);
    XMapWindow(dis, win);
	XFlush(dis);
	do {
		XNextEvent(dis, &ev);
	} while(ev.type!=Expose);

	pm = None;
	pt = 1;
	printf("%s\n", argv[pt]);
	if (load(&pm, win, argv[pt++], &w, &h)!=0) {
		printf("error %s\n", argv[pt-1]);
	} else {
		XResizeWindow(dis, win, w, h);
		XCopyArea(dis, pm, win, gc, 0, 0, w, h, 0, 0);
	}
	
	while(1) {
		if (XCheckMaskEvent(dis,
							ExposureMask | KeyPressMask | ButtonPressMask,
							&ev)) {
			if(ev.type==KeyPress || ev.type==ButtonPress) {
				if (pt==argc) break;
				if (pm!=None && pt<argc) {
					XFreePixmap(dis, pm);
				}
				if (pt<argc) {
					printf("%s\n", argv[pt]);
					if (load(&pm, win, argv[pt++], &w, &h)!=0) {
						printf("error %s\n", argv[pt-1]);
					} else {
						XResizeWindow(dis, win, w, h);
						XCopyArea(dis, pm, win, gc, 0, 0, w, h, 0, 0);
					}
				}
			}
			else if(ev.type==Expose) {
				if (pm!=None) {
					XCopyArea(dis, pm, win, gc, 0, 0, w, h, 0, 0);
				}
			}
		}
	}

	XFreeGC(dis, gc);
	if (pm!=None) XFreePixmap(dis, pm);
	return 0;
}


